import igraph_testing as ig
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import numpy as np
import os
import descriptors
import sys

def main():

    filename = sys.argv[1]
    # dimension = sys.argv[2]
    graph_type = sys.argv[2]
    functionality = sys.argv[3]

    g,is_2D = ig.generateGraph(filename)
    fg = ig.filterGraph(g)


    if functionality == 'visuals':
        if is_2D == True:
            if graph_type == 'g':
                ig.visual2D(g,'graph')
                print("Plot displayed.")
            if graph_type == 'fg':
                ig.visual2D(fg,'filtered')
                print("Plot displayed.")
        else:
            if graph_type == 'g':
                ig.visual3D(g)
            if graph_type == 'fg':
                ig.visual3D(fg)

    if functionality == 'descriptors':
        print(descriptors.desciptors(g))

    
    if functionality == 'cc':
        print(ig.connectedComponents(g))


if __name__ == '__main__':
    main()






from . import igraph_testing as ig
import time 
import tracemalloc
import csv

def functionRuntime(count,function, *argv):
    """
    Measures the average runtime of a function over a specified number of executions.

    Args:
        count (int): The number of times to execute the function.
        function (Callable): The function to measure the runtime for.
        *argv: Arguments to be passed to the function.

    Returns:
        float: The average execution time in seconds across the `count` executions.
    """
    totaltime = 0
    
    for x in range(count):
        startTime = time.time()
        function(*argv)
        endTime = time.time()
        timeTaken = endTime - startTime
        totaltime += timeTaken

    avgExecution = totaltime / count

    return avgExecution

def functionMemory(function, *argv):
    """
    Measures the peak memory usage of a function during its execution.

    Args:
        function (Callable): The function to measure memory usage for.
        *argv: Arguments to be passed to the function.

    Returns:
        int: The memory used in bytes during the function's execution.
    """
    tracemalloc.start()
    function(*argv)
    stats = tracemalloc.get_traced_memory()
    tracemalloc.stop()
    stats = stats[1] - stats[0]
    
    return stats

def csvMaker(fileName, n, dim, count, graphGen, graphGenPar,graphFilt, graphFiltPar,shortPath, shortPathPar):
    """
    Runs multiple graph-related functions, records their runtime and memory usage,
    and stores the results in a CSV file.

    Args:
        fileName (str): The name of the CSV file to write results to.
        n (int): The base size parameter of the graph.
        dim (int): The dimension to compute the total nodes as n^dim.
        count (int): The number of times to run each function for benchmarking.
        graphGen (Callable): Function to generate a graph.
        graphGenPar (tuple): Parameters for the graph generation function.
        graphFilt (Callable): Function to filter the graph.
        graphFiltPar (tuple): Parameters for the graph filtering function.
        shortPath (Callable): Function to compute the shortest path in the graph.
        shortPathPar (tuple): Parameters for the shortest path function.

    Returns:
        None: The function writes results to the specified CSV file.
    """
    row = [n,(n**dim)]
    totalTime = 0
    totalMem = 0

    graphGenRuntime = functionRuntime(count,graphGen,*graphGenPar)
    graphFiltRuntime = functionRuntime(count,graphFilt,*graphFiltPar)
    shortPathRuntime = functionRuntime(count,shortPath,*shortPathPar)

    totalTime = graphGenRuntime + graphFiltRuntime + shortPathRuntime
    totalTime = round(totalTime,20)
    
    graphGenMem = functionMemory(graphGen,*graphGenPar)
    graphFiltMem = functionMemory(graphFilt,*graphFiltPar)
    shortPathMem = functionMemory(shortPath,*shortPathPar)

    totalMem = graphGenMem + graphFiltMem + shortPathMem
    totalMem = round(totalMem,20)

    row.append(graphGenRuntime)
    row.append(graphFiltRuntime)
    row.append(shortPathRuntime)
    row.append(totalTime)
    row.append(graphGenMem)
    row.append(graphFiltMem)
    row.append(shortPathMem)
    row.append(totalMem)

    with open(fileName, 'a', newline = "\n") as file:
        writer = csv.writer(file)
        writer.writerow(row)



import igraph_testing as ig
import time
import tracemalloc
import csv
import os

def writeRow(txt):
    row = []
    finalTotal = 0
    total = 0
    g = ig.lattice(txt)
    for i in range(5):
        start = time.time()
        g=ig.lattice(txt)
        total+=time.time()-start
    total = total/5
    finalTotal +=total
    row.append(total)
    total = 0

    g_filtered = ig.filterGraph(g)
    for i in range(5):
        start = time.time()
        g_filtered = ig.filterGraph(g)
        total+=time.time()-start
    total = total / 5
    finalTotal+=total
    row.append(total)
    total = 0

    for i in range(5):
        start = time.time()
        bfs_paths=ig.shortest_path(g_filtered)
        total+=time.time()-start
    total = total / 5
    finalTotal += total
    row.append(total)
    row.append(finalTotal)
    tracemalloc.start()
    g=ig.lattice(txt)
    g_filtered = ig.filterGraph(g)
    bfs_paths=ig.shortest_path(g_filtered)
    stats = tracemalloc.get_traced_memory()
    tracemalloc.stop()
    stats = stats[1] - stats[0]
    row.append(stats)
    return row

file_exists = os.path.exists('out.csv')
row = writeRow("2D-testFile/testFile-10-2D.txt")
with open('out.csv', mode='a' if file_exists else 'w', newline='') as file:
    writer = csv.writer(file)
    if not file_exists:
        writer.writerow(['creation(s)','filtering(s)','bfs(s)','overall runtime(s)','whole memory usage(byte)'])
    writer.writerow(row)


import igraph_testing as ig
import descriptors as ds
import os
import fpdf
import numpy as np
from PIL import Image
import webbrowser
import argparse

current_dir = os.getcwd()
data_path = f"{current_dir}/graspi_igraph/data/"
descriptors_path = f"{current_dir}/graspi_igraph/descriptors/"
image_path = f"{current_dir}/graspi_igraph/images/"
results_path = f"{current_dir}/graspi_igraph/results/"
test_files = [os.path.splitext(file)[0] for file in os.listdir(data_path)]
epsilon = 1e-5

def generate_image(filename):
    file_path = data_path + filename + ".txt"
    matrix = []
    with open(file_path, "r") as f:
        lines = f.readlines()
        for line in lines[1:]:
            row = []
            line = line.strip().split(" ")
            for char in line:
                row.append(int(char))
            matrix.append(row)
    matrix_array = np.array(matrix, dtype=np.uint8)
    image = Image.fromarray(matrix_array * 255, mode="L")
    bw_image = image.convert("1")
    bw_image.save(image_path + filename + ".png")
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("file_type", choices=["txt", "pdf"])
    args = parser.parse_args()

    if args.file_type == "txt":
        PDF = False
    else:
        PDF = True

    pdf = None

    if PDF:
        pdf = fpdf.FPDF()
        pdf.set_font("Arial", size=12)
        print("Generating PDF")

    print("Generating Text Files")

    for test_file in test_files:
        print(f"Executing {test_file}")
        if PDF:
            pdf.add_page()

        g, is_2D = ig.generateGraph(data_path + test_file + ".txt")
        stats = ds.desciptors(g)

        if PDF:
            pdf.cell(200, 8, txt=f"{test_file} Results", ln=True, align="L")

        with open(results_path + "descriptors-" + test_file + ".txt", "w") as txt:
            txt.write(f"{test_file} Results\n")
            with open(descriptors_path + "descriptors." + test_file + ".log") as f:
                for line in f:
                    stat = line.strip().split(" ")
                    try:
                        if stats.get(stat[0], -1) != -1 and stats.get(stat[0], -1) == int(stat[1]):
                            txt.write(f"{stat[0]} passed - {stats.get(stat[0])} is the same as expected {stat[1]}\n")
                            if PDF:
                                pdf.cell(200, 8, txt=f"{stat[0]} passed - {stats.get(stat[0])} is the same as expected {stat[1]}", ln=True, align="L")
                        elif stats.get(stat[0], -1) != -1 and stats.get(stat[0], -1) != int(stat[1]):
                            txt.write(f"{stat[0]} failed - {stats.get(stat[0])} is not the same as expected {stat[1]}\n")
                            if PDF:
                                pdf.cell(200, 8, txt=f"{stat[0]} failed - {stats.get(stat[0])} is not the same as expected {stat[1]}", ln=True, align="L")
                    except ValueError:
                        if stats.get(stat[0], -1) != -1 and abs(stats.get(stat[0], -1) - float(stat[1])) < epsilon:
                            txt.write(f"{stat[0]} passed - {stats.get(stat[0])} is the same as expected {stat[1]}\n")
                            if PDF:
                                pdf.cell(200, 8, txt=f"{stat[0]} passed - {stats.get(stat[0])} is the same as expected {stat[1]}", ln=True, align="L")
                        elif stats.get(stat[0], -1) != -1 and stats.get(stat[0], -1) != float(stat[1]):
                            txt.write(f"{stat[0]} failed - {stats.get(stat[0])} is not the same as expected {stat[1]}\n")
                            if PDF:
                                pdf.cell(200, 8,txt=f"{stat[0]} failed - {stats.get(stat[0])} is not the same as expected {stat[1]}",ln=True, align="L")
            txt.write(f"{stats}\n")

        if PDF:
            pdf.multi_cell(200, 8, txt=f"{stats}", align="L")
            generate_image(test_file)
            image_file = image_path + test_file + ".png"
            pdf.image(image_file)
    print("Text Files Generated")

    if PDF:
        pdf.output(f"{current_dir}/graspi_igraph/test_results.pdf")
        print("PDF Generated")
        webbrowser.open_new_tab(f"{current_dir}/graspi_igraph/test_results.pdf")

if __name__ == "__main__":
    main()


from setuptools import setup, find_packages

setup(
    name = "graspi_igraph",
    author = "Wenqi Zheng",
    author_email = "wenqizhe@buffalo.edu",
    version = "0.0.2.0",
    description = "Utilize Python-igraph to produce similar functionality as GraSPI",
    packages = find_packages(),
    classifiers = ["Programming Language :: Python"],
    install_requires=[
        "igraph",
        "matplotlib",
        "numpy"
    ],
    python_requires = ">=3.7"
    
)